@@ -65,8 +65,8 @@ class Octree:
     self.nnum = torch.zeros(num, dtype=torch.int32)
     self.nnum_nempty = torch.zeros(num, dtype=torch.int32)
 
-    # the following properties are valid after `merge_octrees` and `build_octree`
-    # TODO: make them available after `octree_grow` and `octree_split`
+    # the following properties are valid after `merge_octrees`.
+    # TODO: make them valid after `octree_grow`, `octree_split` and `build_octree`
     batch_size = self.batch_size
     self.batch_nnum = torch.zeros(num, batch_size, dtype=torch.int32)
     self.batch_nnum_nempty = torch.zeros(num, batch_size, dtype=torch.int32)
@@ -148,16 +148,22 @@ class Octree:
 
     Args:
       point_cloud (Points): The input point cloud.
+
+    .. note::
+      Currently, the batch size of the point cloud must be 1.
     '''
 
     self.device = point_cloud.device
+    assert point_cloud.batch_size == self.batch_size, 'Inconsistent batch_size'
 
-    # normalize points from [-1, 1] to [0, 2^depth]
+    # normalize points from [-1, 1] to [0, 2^depth]. #[Lable:S]
     scale = 2 ** (self.depth - 1)
     points = (point_cloud.points + 1.0) * scale
 
     # get the shuffled key and sort
-    key = xyz2key(points[:, 0], points[:, 1], points[:, 2], depth=self.depth)
+    x, y, z = points[:, 0], points[:, 1], points[:, 2]
+    b = None if self.batch_size == 1 else point_cloud.batch_id.view(-1)
+    key = xyz2key(x, y, z, b, self.depth)
     node_key, idx, counts = torch.unique(
         key, sorted=True, return_inverse=True, return_counts=True)
 
@@ -173,7 +179,7 @@ class Octree:
           pkey, return_inverse=True, return_counts=True)
 
       # augmented key
-      key = pkey.unsqueeze(-1) * 8 + torch.arange(8, device=self.device)
+      key = (pkey.unsqueeze(-1) << 3) + torch.arange(8, device=self.device)
       self.keys[d] = key.view(-1)
       self.nnum[d] = key.numel()
       self.nnum_nempty[d] = node_key.numel()
@@ -187,24 +193,25 @@ class Octree:
       self.children[d] = children
 
       # cache pkey for the next iteration
-      node_key = pkey
+      # Use `pkey >> 45` instead of `pkey >> 48` in L199 since pkey is already
+      # shifted to the right by 3 bits in L177
+      node_key = pkey if self.batch_size == 1 else  \
+          ((pkey >> 45) << 48) | (pkey & ((1 << 45) - 1))
 
     # set the children for the layer full_layer,
     # now the node_keys are the key for full_layer
     d = self.full_depth
     children = -torch.ones_like(self.children[d])
-    children[node_key] = torch.arange(
+    nempty_idx = node_key if self.batch_size == 1 else  \
+        ((node_key >> 48) << (3 * d)) | (node_key & ((1 << 48) - 1))
+    children[nempty_idx] = torch.arange(
         node_key.numel(), dtype=torch.int32, device=self.device)
     self.children[d] = children
     self.nnum_nempty[d] = node_key.numel()
 
-    # set the batch_nnum
-    self.batch_nnum[:, 0] = self.nnum
-    self.batch_nnum_nempty[:, 0] = self.nnum_nempty
-
     # average the signal for the last octree layer
     d = self.depth
-    points = scatter_add(points, idx, dim=0)  # here points is rescaled in L84
+    points = scatter_add(points, idx, dim=0)  # points is rescaled in [Lable:S]
     self.points[d] = points / counts.unsqueeze(1)
     if point_cloud.normals is not None:
       normals = scatter_add(point_cloud.normals, idx, dim=0)
@@ -219,7 +226,7 @@ class Octree:
     r''' Builds the full octree, which is essentially a dense volumetric grid.
 
     Args:
-      depth (int): The depth of the octree. 
+      depth (int): The depth of the octree.
       update_neigh (bool): If True, construct the neighborhood indices.
     '''
 
@@ -270,12 +277,12 @@ class Octree:
     self.nnum_nempty[depth] = nnum_nempty
 
   def octree_grow(self, depth: int, update_neigh: bool = True):
-    r''' Grows the octree and updates the relevant properties. And in most 
+    r''' Grows the octree and updates the relevant properties. And in most
     cases, call :func:`Octree.octree_split` to update the splitting status of
     the octree before this function.
 
     Args:
-      depth (int): The depth of the octree. 
+      depth (int): The depth of the octree.
       update_neigh (bool): If True, construct the neighborhood indices.
     '''
 
@@ -347,7 +354,7 @@ class Octree:
     r''' Searches the octree nodes given the query points.
 
     Args:
-      query (torch.Tensor): The coordinates of query points with shape 
+      query (torch.Tensor): The coordinates of query points with shape
           :obj:`(N, 4)`. The first 3 channels of the coordinates are :obj:`x`,
           :obj:`y`, and :obj:`z`, and the last channel is the batch index. Note
           that the coordinates must be in range :obj:`[0, 2^depth)`.
@@ -364,7 +371,7 @@ class Octree:
 
     Args:
       query (torch.Tensor): The keys of query points with shape :obj:`(N,)`,
-          which are computed from the coordinates of query points. 
+          which are computed from the coordinates of query points.
       depth (int): The depth of the octree layer. nemtpy (bool): If true, only
           searches the non-empty octree nodes.
     '''
